@@ -216,106 +216,8 @@ Perl_cast_uv(NV f)
   return f > 0 ? UV_MAX : 0 /* NaN */;
 }
 
-/*
-=for apidoc      grok_bin
-=for apidoc_item grok_hex
-=for apidoc_item grok_oct
-
-These each convert a string representing a number to numeric form.  The
-number is binary in C<grok_bin>, octal in C<grok_oct>, and hexadecimal in
-C<grok_hex>.
-
-On entry C<start> and C<*len_p> give the string to scan, C<*flags> gives
-conversion flags, and C<result> should be C<NULL> or a pointer to an NV.  The
-scan stops at the end of the string, or at just before the first invalid
-character.  Unless C<PERL_SCAN_SILENT_ILLDIGIT> is set in C<*flags>,
-encountering an invalid character (except NUL) will also trigger a warning.  On
-return C<*len_p> is set to the length of the scanned string, and C<*flags>
-gives output flags.
-
-If the value is S<E<lt>= C<UV_MAX>>, it is returned as a UV, the output flags are
-clear, and nothing is written to C<*result>.  If the value is S<E<gt>
-C<UV_MAX>>:
-
-=over
-
-=item *
-
-C<UV_MAX> is returned
-
-=item *
-
-C<PERL_SCAN_GREATER_THAN_UV_MAX> is set in C<*flags>.
-
-=item *
-
-If C<result> is not null, an approximation of the correct value is written
-into C<*result> (which is an NV).
-
-=back
-
-Unless C<PERL_SCAN_DISALLOW_PREFIX> is set in C<*flags> on entry:
-
-=over
-
-=item For C<grok_bin>
-
-the input string may optionally be prefixed with C<"0b"> or plain C<"b">
-
-=item For C<grok_hex>
-
-the input string may optionally be prefixed with C<"0x"> or plain C<"x">
-unless
-
-=item For C<grok_oct>
-
-this flag is ignored; there is no optional prefix.  The typical C<0> prefix is
-just part of the number.
-
-=back
-
-If C<PERL_SCAN_ALLOW_UNDERSCORES> is set in C<*flags> then any or all pairs of
-digits may be separated from each other by a single underscore, and also a
-single leading underscore is accepted.
-
-=for apidoc Amnh||PERL_SCAN_ALLOW_UNDERSCORES
-=for apidoc Amnh||PERL_SCAN_DISALLOW_PREFIX
-=for apidoc Amnh||PERL_SCAN_GREATER_THAN_UV_MAX
-=for apidoc Amnh||PERL_SCAN_SILENT_ILLDIGIT
-
-=cut
-
-Not available externally yet because experimental is
-C<PERL_SCAN_SILENT_NON_PORTABLE which suppresses any message for non-portable
-numbers that are still valid on this platform.
- */
-
-UV
-Perl_grok_bin(pTHX_ const char *start, STRLEN *len_p, I32 *flags, NV *result)
-{
-    PERL_ARGS_ASSERT_GROK_BIN;
-
-    return grok_bin(start, len_p, flags, result);
-}
-
-UV
-Perl_grok_hex(pTHX_ const char *start, STRLEN *len_p, I32 *flags, NV *result)
-{
-    PERL_ARGS_ASSERT_GROK_HEX;
-
-    return grok_hex(start, len_p, flags, result);
-}
-
-UV
-Perl_grok_oct(pTHX_ const char *start, STRLEN *len_p, I32 *flags, NV *result)
-{
-    PERL_ARGS_ASSERT_GROK_OCT;
-
-    return grok_oct(start, len_p, flags, result);
-}
-
-STATIC void
-S_output_non_portable(pTHX_ const U8 base)
+void
+Perl_output_non_portable(pTHX_ const U8 base)
 {
     /* Display the proper message for a number in the given input base not
      * fitting in 32 bits */
@@ -393,11 +295,13 @@ Perl_grok_bin_oct_hex(pTHX_ const char * const start,
         }
     }
 
-    const char * const s0 = s; /* Where the significant digits start */
-    UV value = 0;
+    const char * const s0 = s;  /* Where the significant digits start */
+    UV value = 0;               /* Running total */
 
-    /* Unroll the loop so that the first 8 digits are branchless except for the
-     * switch.  A ninth hex one overflows a 32 bit word. */
+    /* Unroll the loop so that numbers with 8 or fewer digits can be handled
+     * with the minimum amount of work.  Anything higher would require extra
+     * overhead to deal with the possibility of generating portability
+     * warnings for numbers above 32 bits, which is reached at 8 hex digits */
   redo_switch:
     switch (e - s) {
       default:
@@ -455,9 +359,14 @@ Perl_grok_bin_oct_hex(pTHX_ const char * const start,
           }
 
           break;
-    }
+    }   /* End of switch on the first so-many characters */
 
-    /* In overflows, this keeps track of how much to multiply the overflowed NV
+    /* The loop below accumulates the integral running total of the result,
+     * digit by digit.  If this total overflows, it is added to an NV
+     * approximation, and the loop starts over, looking at the next batch of
+     * digits, until they overflow, and so on.
+     *
+     * In overflows, this keeps track of how much to multiply the overflowed NV
      * by as we continue to parse the remaining digits */
     NV factor = 0.0;
 
@@ -465,6 +374,11 @@ Perl_grok_bin_oct_hex(pTHX_ const char * const start,
     NV value_nv = 0;
     const PERL_UINT_FAST8_T base = 1 << shift;  /* 2, 8, or 16 */
 
+    /* As long as the running total is less than this, the next digit will
+     * fit. */
+    UV max_div = UV_MAX >> shift;
+
+    /* Loop through the characters */
     for (; s < e; s++) {
         if (generic_isCC_(*s, class_bit)) {
             /* Write it in this wonky order with a goto to attempt to get the
@@ -473,18 +387,13 @@ Perl_grok_bin_oct_hex(pTHX_ const char * const start,
                (khw suspects that adding a LIKELY() just above would do the
                same thing) */
           redo: ;
-
-            /* Make room for the next digit */
-            UV tentative_value = value << shift;
-
-            /* If shiftng back doesn't yield the previous value, it was
-             * because a bit got shifted off the left end, so overflowed.
-             * But if it worked, add the new digit. */
-            if (LIKELY((tentative_value >> shift) == value)) {
-                value = tentative_value | XDIGIT_VALUE(*s);
-                    /* Note XDIGIT_VALUE() is branchless, works on binary
-                     * and octal as well, so can be used here, without
-                     * slowing those down */
+            /* If there is room for this digit, accumulate it and repeat */
+            if (LIKELY(value <= max_div)) {
+                /* Note XDIGIT_VALUE() is branchless, works on binary and
+                 * octal as well, so can be used here, without noticeably
+                 * slowing those down (it does have unnecessary shifts, ANDSs,
+                 * and additions for those) */
+                value = (value << shift) | XDIGIT_VALUE(*s);
                 factor *= base;
                 continue;
             }
@@ -498,37 +407,24 @@ Perl_grok_bin_oct_hex(pTHX_ const char * const start,
             value_nv += (NV) value;
 
             /* Then we keep accumulating digits, until all are parsed.  We
-             * start over using the current input value.  This will be added to
-             * 'value_nv' eventually, either when all digits are gone, or we
-             * have overflowed this fresh start. */
+             * start over using the current input value as the initial digit.
+             * This will be added to 'value_nv' eventually, either when all
+             * digits are gone, or we have overflowed this fresh start.  This
+             * method uses the fewest floating point multiplications possible,
+             * losing the least precision. */
             value = XDIGIT_VALUE(*s);
             factor = base;
-
-            if (! overflowed) {
-                overflowed = TRUE;
-                if (input_flags & PERL_SCAN_SILENT_OVERFLOW) {
-                    *flags |= PERL_SCAN_SILENT_OVERFLOW;
-                }
-                else if (ckWARN_d(WARN_OVERFLOW)) {
-                    warner(packWARN(WARN_OVERFLOW),
-                           "Integer overflow in %s number",
-                           (base == 16) ? "hexadecimal"
-                                        : (base == 2)
-                                          ? "binary"
-                                          : "octal");
-                }
-            }
+            overflowed = TRUE;
             continue;
-        }
+        } /* End of handling legal digit */
 
-        if (   *s == '_'
+        /* Handle non-trailing underscores when those are accepted */
+        if (   UNLIKELY(*s == '_')
             && s < e - 1
             && allow_underscores
             && generic_isCC_(s[1], class_bit)
-
-                /* Don't allow a leading underscore if the only-medial bit is
-                 * set */
             && (   LIKELY(s > s0)
+                   /* Including initial underscores if those are accepted */
                 || UNLIKELY(! (  input_flags
                                & PERL_SCAN_ALLOW_MEDIAL_UNDERSCORES_ONLY))))
         {
@@ -537,7 +433,7 @@ Perl_grok_bin_oct_hex(pTHX_ const char * const start,
             /* To get here with the value so-far being 0 means we've only had
              * leading zeros, then an underscore.  We can continue with the
              * branchless switch() instead of this loop */
-            if (value == 0) {
+            if (UNLIKELY(value == 0)) {
                 goto redo_switch;
             }
             else {
@@ -545,63 +441,89 @@ Perl_grok_bin_oct_hex(pTHX_ const char * const start,
             }
         }
 
-        if (*s) {
-            if (   ! (input_flags & PERL_SCAN_SILENT_ILLDIGIT)
-                &&    ckWARN(WARN_DIGIT))
-            {
-                if (base != 8) {
-                    warner(packWARN(WARN_DIGIT),
-                           "Illegal %s digit '%c' ignored",
-                           ((base == 2)
-                            ? "binary"
-                            : "hexadecimal"),
-                           *s);
-                }
-                else if (isDIGIT(*s)) { /* octal base */
+        /* We get here when done with the parse, or it got interrupted by a
+         * non-digit or a digit that is outside the bounds of the base, like a
+         * digit 2 in a binary number.  In either case, we are done with the
+         * loop */
+        break;
+    }   /* End of parsing loop */
 
-                    /* Allow \octal to work the DWIM way (that is, stop
-                     * scanning as soon as non-octal characters are seen,
-                     * complain only if someone seems to want to use the digits
-                     * eight and nine.  Since we know it is not octal, then if
-                     * isDIGIT, must be an 8 or 9). */
-                    warner(packWARN(WARN_DIGIT),
-                           "Illegal octal digit '%c' ignored", *s);
-                }
-            }
+    bool do_non_portable_output = false;
 
-            if (input_flags & PERL_SCAN_NOTIFY_ILLDIGIT) {
-                *flags |= PERL_SCAN_NOTIFY_ILLDIGIT;
-            }
+    if (UNLIKELY(overflowed)) {
+
+        /* Calculate the final overflow approximation */
+        value_nv *= factor;
+        value_nv += (NV) value;
+
+        *flags |= PERL_SCAN_GREATER_THAN_UV_MAX
+               |  PERL_SCAN_SILENT_NON_PORTABLE;
+
+        if (result)
+            *result = value_nv;
+
+        if (input_flags & PERL_SCAN_SILENT_OVERFLOW) {
+            *flags |= PERL_SCAN_SILENT_OVERFLOW;
+        }
+        else if (ckWARN_d(WARN_OVERFLOW)) {
+            warner(packWARN(WARN_OVERFLOW),
+                    "Integer overflow in %s number",
+                    (base == 16) ? "hexadecimal"
+                                : (base == 2)
+                                    ? "binary"
+                                    : "octal");
         }
 
-        break;
+        value = UV_MAX;
+        do_non_portable_output = true;
     }
-
-    *len_p = s - start;
-
-    if (LIKELY(! overflowed)) {
+    else {
 #if UVSIZE > 4
         if (UNLIKELY(value > 0xffffffff)) {
             if (! (input_flags & PERL_SCAN_SILENT_NON_PORTABLE)) {
-                output_non_portable(base);
+                do_non_portable_output = true;
             }
             *flags |= PERL_SCAN_SILENT_NON_PORTABLE;
         }
 #endif
-        return value;
     }
 
-    /* Overflowed: Calculate the final overflow approximation */
-    value_nv *= factor;
-    value_nv += (NV) value;
+    if (s < e && *s) {  /* *s is to keep a terminating NUL from warning */
+        if (   ! (input_flags & PERL_SCAN_SILENT_ILLDIGIT)
+            &&    ckWARN(WARN_DIGIT))
+        {
+            if (base != 8) {
+                warner(packWARN(WARN_DIGIT),
+                        "Illegal %s digit '%c' ignored",
+                        ((base == 2)
+                        ? "binary"
+                        : "hexadecimal"),
+                        *s);
+            }
+            else if (isDIGIT(*s)) { /* octal base */
 
-    output_non_portable(base);
+                /* Allow \octal to work the DWIM way (that is, stop scanning
+                 * as soon as non-octal characters are seen, complain only if
+                 * someone seems to want to use the digits eight and nine.
+                 * Since we know it is not octal, then if isDIGIT, must be an
+                 * 8 or 9). khw: XXX why not DWIM for other bases as well? */
+                warner(packWARN(WARN_DIGIT),
+                        "Illegal octal digit '%c' ignored", *s);
+            }
+        }
 
-    *flags |= PERL_SCAN_GREATER_THAN_UV_MAX
-           |  PERL_SCAN_SILENT_NON_PORTABLE;
-    if (result)
-        *result = value_nv;
-    return UV_MAX;
+        if (input_flags & PERL_SCAN_NOTIFY_ILLDIGIT) {
+            *flags |= PERL_SCAN_NOTIFY_ILLDIGIT;
+        }
+    }
+
+    if (UNLIKELY(do_non_portable_output)) {
+        output_non_portable(base);
+    }
+
+    /* s here points to e or to the first illegal character */
+    *len_p = s - start;
+    return value;
 }
 
 /*
